/*
 * Copyright (c) 1999 Greg Haerr <greg@censoft.com>
 *
 * 8bpp Linear Video Driver for MicroWindows
 *
 * Inspired from Ben Pfaff's BOGL <pfaffben@debian.org>
 */
/*#define NDEBUG*/
#include <assert.h>
#include <string.h>
#include "../device.h"
#include "fb.h"

/* Set pixel at x, y, to pixelval c*/
void
linear8_drawpixel(ADDR8 addr, COORD x, COORD y, PIXELVAL c)
{
	assert (x >= 0 && x < XRES);
	assert (y >= 0 && y < YRES);
	assert (c >= 0 && c < NCOLORS);

	++drawing;
	if(gr_mode == MODE_XOR)
		addr[x + y * LINELEN] ^= c;
	else
		addr[x + y * LINELEN] = c;
	--drawing;
}

/* Read pixel at x, y*/
PIXELVAL
linear8_readpixel(ADDR8 addr, COORD x, COORD y)
{
	assert (x >= 0 && x < XRES);
	assert (y >= 0 && y < YRES);

	return addr[x + y * LINELEN];
}

/* Draw horizontal line from x1,y to x2,y not including final point*/
void
linear8_drawhorzline(ADDR8 addr, COORD x1, COORD x2, COORD y, PIXELVAL c)
{
	assert (x1 >= 0 && x1 < XRES);
	assert (x2 >= 0 && x2 <= XRES);
	assert (x2 >= x1);
	assert (y >= 0 && y < YRES);
	assert (c >= 0 && c < NCOLORS);

	++drawing;
	addr += x1 + y * LINELEN;
	if(gr_mode == MODE_XOR) {
		while(x1++ < x2)
			*addr++ ^= c;
	} else
		memset(addr, c, x2 - x1);
	--drawing;
}

/* Draw a vertical line from x,y1 to x,y2 not including final point*/
void
linear8_drawvertline(ADDR8 addr, COORD x, COORD y1, COORD y2, PIXELVAL c)
{
	assert (x >= 0 && x < XRES);
	assert (y1 >= 0 && y1 < YRES);
	assert (y2 >= 0 && y2 <= YRES);
	assert (y2 >= y1);
	assert (c >= 0 && c < NCOLORS);

	++drawing;
	addr += x + y1 * LINELEN;
	if(gr_mode == MODE_XOR)
		while(y1++ < y2) {
			*addr ^= c;
			addr += LINELEN;
		}
	else
		while(y1++ < y2) {
			*addr = c;
			addr += LINELEN;
		}
	--drawing;
}

void
linear8_blit(ADDR8 dstaddr, COORD destx, COORD desty, COORD w, COORD h,
	ADDR8 srcaddr, COORD srcx, COORD srcy)
{
	ADDR8	dst;
	ADDR8	src;

	assert (destx >= 0 && destx < XRES);
	assert (desty >= 0 && desty < YRES);
	assert (w > 0);
	assert (h > 0);
	assert (srcx >= 0 && srcx <= XRES);
	assert (srcy >= 0 && srcy < YRES);

	++drawing;
	// FIXME: LINELEN will change on offscreen bitmaps
	dst = dstaddr + destx + desty * LINELEN;
	src = srcaddr + srcx + srcy * LINELEN;
	while(--h >= 0) {
		memcpy(dst, src, w);
		dst += LINELEN;
		src += LINELEN;
	}
	--drawing;
}

/* Return framebuffer mmap size*/
int
linear8_init(void)
{
	return YRES * LINELEN * sizeof(unsigned char);
}

FBENTRY fblinear8 = {
	(void *)linear8_init,
	(void *)linear8_drawpixel,
	(void *)linear8_readpixel,
	(void *)linear8_drawhorzline,
	(void *)linear8_drawvertline,
	(void *)linear8_blit
};
